from saruca_uuv.comprehend_system_layer.Classes import VehicleComprehendLayer
from saruca_uuv.comprehend_system_layer.Methods import localization
from saruca_uuv.comprehend_system_layer.Methods import state
from saruca_uuv.control_system_layer.Classes import ControlLayer
from saruca_uuv.control_system_layer.Methods import assignment
from saruca_uuv.control_system_layer.Methods import behaviour
from saruca_uuv.control_system_layer.Methods import command
from saruca_uuv.control_system_layer.Methods import engineBreakdown
from saruca_uuv.control_system_layer.Methods import orbit
from saruca_uuv.sensing_system_layer.Classes import EntityLayer
from saruca_uuv.sensing_system_layer.Methods import imageProcessing
from saruca_uuv.sensing_system_layer.Methods import variables



# state

from ..Classes.VehicleComprehendLayer import VehicleComprehendLayer

speed = 0
accel = 0.0
directory = ""
angle = 0.0
poolSize = [(25, 50, 2), (12.5, 25, 2)]


def state():
    vehicleState = VehicleComprehendLayer(speed, accel, directory, angle, poolSize)
    if vehicleState.getPoolSize() == [(25, 50, 2), (25, 50, 2)]:
        if vehicleState.getSpeed() > 15 and vehicleState.getAccel() > 2.0:
            return "highAccel"
        else:
            return "normalAccel"
    else:
        return "normalAccel"



# image processing
# variables

class EntityLayer:
    __boundingBox = []
    __otherVariables = []
    __objectType = ''
    __objectDistance = 0
    __positionVector = (0, 0)

    def __init__(self,
                 boundingBoxes=None,
                 objectType=None,
                 objectDistance=None,
                 otherVariables=None,
                 positionVector=None):
        if boundingBoxes is None:
            self.__boundingBox = []
        else:
            for __boundingBox in boundingBoxes:
                self.__boundingBox = boundingBoxes
        if otherVariables is None:
            self.__otherVariables = []
        else:
            for __otherVariables in otherVariables:
                self.__otherVariables = otherVariables
        if objectType is None:
            self.__objectType = ''
        else:
            self.__objectType = objectType
        if objectDistance is None:
            self.__objectDistance = 0
        else:
            self.__objectDistance = objectDistance
        if positionVector is None:
            self.__positionVector = (0, 0)
        else:
            self.__positionVector = positionVector

    def getBoundingBoxes(self):
        return self.__boundingBox

    def getOtherVariables(self):
        return self.__otherVariables

    def getObjectType(self):
        return self.__objectType

    def getObjectDistance(self):
        return self.__objectDistance

    def getPositionVector(self):
        return self.__positionVector



# state
# localization

class VehicleComprehendLayer:
    __speed = 0
    __accel = 0.0
    __directory = ""
    __angle = 0.0
    __poolSize = [(25, 50, 2), (12.5, 25, 2)]

    def __init__(self,
                 speed=None,
                 accel=None,
                 directory=None,
                 angle=None,
                 poolSize=None):
        if speed is None:
            self.__speed = 0
        else:
            self.__speed = speed
        if accel is None:
            self.__accel = 0.0
        else:
            self.__accel = accel
        if directory is None:
            self.__directory = ""
        elif directory == "forward" or directory == "ileri":
            self.__directory = "forward"
        else:
            self.__directory = directory
        if angle is None:
            self.__angle = 0.0
        else:
            self.__angle = angle
        if poolSize is None:
            self.__poolSize = [(25, 50, 2), (25, 50, 2)]
        else:
            self.__poolSize = poolSize

    def getSpeed(self):
        return self.__speed

    def getAccel(self):
        return self.__accel

    def getDirectory(self):
        return self.__directory

    def getAngle(self):
        return self.__angle

    def getPoolSize(self):
        return self.__poolSize



# object data
# distance data
import cv2
import numpy as np
from ..Classes.EntityLayer import EntityLayer


def circleAndRectangleDetection():
    """
        Detection
    """
    captureFrame = cv2.VideoCapture("<VIDEO PATH>")
    while True:
        ret, frame = captureFrame.read()

        # frame = cv2.flip(frame, 1)
        # frame = cv2.resize(frame, (960, 720))

        frameWidth = frame.shape[1]
        frameHeight = frame.shape[0]

        frameBlob = cv2.dnn.blobFromImage(frame, 1 / 255, (416, 416), swapRB=True, crop=False)

        labels = ["rectangle", "circle"]

        colors = ["0, 0, 255", "0, 0, 255", "255, 0, 0", "255, 255, 0", "0, 255, 0"]
        colors = [np.array(color.split(",")).astype("int") for color in colors]
        colors = np.array(colors)
        colors = np.tile(colors, (18, 1))

        model = cv2.dnn.readNetFromDarknet("<CFG FILE PATH>", "<WEIGHTS FILE PATH>")

        layers = model.getLayerNames()
        outputLayer = [layers[layer[0] - 1] for layer in model.getUnconnectedOutLayers()]

        model.setInput(frameBlob)

        detectionLayers = model.forward(outputLayer)

        idsList = []
        boxesList = []
        confidencesList = []

        for detectionLayer in detectionLayers:
            for objectDetection in detectionLayer:
                scores = objectDetection[5:]
                predictedId = np.argmax(scores)
                confidence = scores[predictedId]

                if confidence > 0.7:
                    label = labels[predictedId]
                    boundingBox = objectDetection[0:4] * np.array([frameWidth, frameHeight, frameWidth, frameHeight])
                    (boxCenterX, boxCenterY, boxWidth, boxHeight) = boundingBox.astype("int")

                    startX = int(boxCenterX - (boxWidth / 2))
                    startY = int(boxCenterY - (boxHeight / 2))

                    idsList.append(predictedId)
                    confidencesList.append(float(confidence))
                    boxesList.append([startX, startY, int(boxWidth), int(boxHeight)])

        maxIds = cv2.dnn.NMSBoxes(boxesList, confidencesList, 0.5, 0.4)

        for maxId in maxIds:
            maxClassId = maxId[0]
            box = boxesList[maxClassId]

            startX = box[0]
            startY = box[1]
            boxWidth = box[2]
            boxHeight = box[3]

            predictedId = idsList[maxClassId]
            label = labels[predictedId]
            confidence = confidencesList[maxClassId]

            endX = startX + boxWidth
            endY = startY + boxHeight

            boxColor = colors[predictedId]
            boxColor = [int(each) for each in boxColor]

            label = "{}: {:.2f}%".format(label, confidence * 100)
            # print("predicted object {}".format(label))

            cv2.rectangle(frame, (startX, startY), (endX, endY), boxColor, 2)
            cv2.rectangle(frame, (startX - 1, startY), (endX + 1, startY - 30), boxColor, -1)
            cv2.putText(frame, label, (startX, startY - 10), cv2.FONT_HERSHEY_SCRIPT_COMPLEX, 0.5, (255, 255, 255), 2)

        if cv2.waitKey(1) & ord("q") == 27:
            break
        cv2.imshow("Tespit Edilen", frame)
    captureFrame.release()
    cv2.destroyAllWindows()


# sample usage
imageObject = EntityLayer((25, 15), "rectangle", 12, "trapezium")
print(imageObject.getBoundingBoxes())
# circleAndRectangleDetection()



# localization

from saruca_uuv.control_system_layer.Methods.assignment import assignment
from saruca_uuv.control_system_layer.Methods.assignment import EntityLayer

vehicleLocation = assignment()


def localization():
    if EntityLayer.getPositionVector() == vehicleLocation:
        return 1
    else:
        return 0




from distutils.core import setup
setup(
  name = 'saruca_uuv',
  packages = ['saruca_uuv'],
  version = '0.1.2',
  license='MIT',
  description = 'Autonomous and manual control methods for UUVs',
  author = 'SEMIH SAHIN',
  author_email = 'esemihsahin@hotmail.com',
  url = 'https://github.com/SARUCA-YazilimEkibi/saruca_uuv',
  keywords = ['UUV', 'AUV', 'UNDERWATER'],
  install_requires=[
          'validators',
          'beautifulsoup4',
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',
    'Intended Audience :: Developers',
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
)